const { basename } = require('path')
const loadConfig = require('../load-config')
const downloadTemplate = require('../download-template')
const getTemplateVersions = require('../get-template-versions')
const throwError = require('../utils/throw-error')
const applyTemplate = require('../apply-template')
const securityCheck = require('../security-check')
const checkParams = require('../check-params')
const prompt = require('../prompt')


module.exports = async(options = {}) => {
  const cwd = options.cwd || process.cwd()
  const name = options.name || basename(cwd)
  const repository = options.repository
  const noDeps = options.noDeps
  let version = options.version

  if (!options.force) await securityCheck(cwd)
  if (version) checkParams.version(version)

  let config = await loadConfig({
    operation: 'init',
    cwd,
    projectName: name,
    templateRepository: repository,
  })

  const versions = await getTemplateVersions(config.template.repository)
  if (version) {
    version = versions.find(v => v.number === version)
    if (!version) {
      throwError([
        'No corresponding template version was found',
        'Please confirm that the version number exists in the tags of the template repository.',
        `Expected template version: ${version}`,
      ].join('\n'))
    }
  } else {
    version = versions[0]
  }


  await downloadTemplate(config.template.repository, version, noDeps)

  config = await config.reload({
    templateVersion: version,
    loadTemplate: true,
  })

  checkParams.engine(config)

  if (config.template.status !== 'loaded') {
    throwError([
      'The template configuration file could not be loaded',
      'Please check the template entry file for errors that make it unloadable',
    ].join('\n'))
  }

  await prompt(config)
  await applyTemplate(config)
  await config.template.hooks('afterInit')
}
